/* Generated automatically. */
static const char configuration_arguments[] = "/var/tmp/portage/sys-devel/gcc-15.2.1_p20260214/work/gcc-15-20260214/configure --host=aarch64-unknown-linux-musl --build=aarch64-unknown-linux-musl --prefix=/usr --bindir=/usr/aarch64-unknown-linux-musl/gcc-bin/15 --includedir=/usr/lib/gcc/aarch64-unknown-linux-musl/15/include --datadir=/usr/share/gcc-data/aarch64-unknown-linux-musl/15 --mandir=/usr/share/gcc-data/aarch64-unknown-linux-musl/15/man --infodir=/usr/share/gcc-data/aarch64-unknown-linux-musl/15/info --with-gxx-include-dir=/usr/lib/gcc/aarch64-unknown-linux-musl/15/include/g++-v15 --disable-silent-rules --disable-dependency-tracking --with-python-dir=/share/gcc-data/aarch64-unknown-linux-musl/15/python --enable-languages=c,c++,fortran --enable-obsolete --enable-secureplt --disable-werror --with-system-zlib --disable-nls --disable-libunwind-exceptions --enable-checking=release --with-bugurl=https://bugs.gentoo.org/ --with-pkgversion='Gentoo Hardened 15.2.1_p20260214 p5' --with-gcc-major-version-only --enable-libstdcxx-time --enable-lto --disable-libstdcxx-pch --enable-linker-build-id --enable-shared --enable-threads=posix --enable-__cxa_atexit --disable-multilib --disable-fixed-point --enable-libgomp --disable-libssp --disable-libada --disable-standard-branch-protection --disable-systemtap --disable-valgrind-annotations --disable-vtable-verify --disable-libvtv --with-zstd --without-isl --enable-default-pie --enable-host-pie --enable-host-bind-now --enable-default-ssp --disable-fixincludes --with-gxx-libcxx-include-dir=/usr/include/c++/v1";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { NULL, NULL} };
